@@ -2,6 +2,7 @@
 # -*- coding: UTF-8 -*-
 from __future__ import annotations
 
+from .exceptions import SilentKillError
 from .libs.color import Color
 from .libs.scriptlocation import ScriptLocation
 
@@ -34,6 +35,7 @@ class Fusion(object):
     print_timestamp = False
     max_filename = 28
 
+    _bundle_pattern = re.compile(r'(fusion_bundle\.js):(\d+)')
     _script_name = Path(__file__).name
     _db_jobs = queue.Queue()
 
@@ -44,6 +46,7 @@ class Fusion(object):
         self.pid = 0
         self.script_trace = {}
         self._modules = []
+        self._ignore_messages = []
         signal.signal(signal.SIGINT, self.signal_handler)
 
         t = threading.Thread(target=Fusion._db_worker, daemon=True)
@@ -63,10 +66,12 @@ class Fusion(object):
         try:
             self.session.detach()
         except:
-            try:
-                self.device.kill(self.pid)
-            except:
-                pass
+            pass
+
+        try:
+            self.device.kill(self.pid)
+        except:
+            pass
 
     def get_device(self):
         try:
@@ -104,25 +109,6 @@ class Fusion(object):
             if v[0] <= loc.get_int_line() <= v[1]
         ]), loc)
 
-    @classmethod
-    def print_message(cls, level: str = "*", message: str = "",
-                      script_location: ScriptLocation = None):
-
-        if Fusion.running is False and Logger.debug_level >= 2:
-            return
-
-        if script_location is None:
-            script_location = ScriptLocation(
-                file_name=Fusion._script_name
-            )
-
-        Logger.print_message(
-            level=level,
-            message=message,
-            script_location=script_location,
-            filename_col_len=Fusion.max_filename
-        )
-
     def load_all_scripts(self):
         self.script_trace = {}
         offset = 1
@@ -149,6 +135,13 @@ class Fusion(object):
 
         for m in self._modules:
             files_js += m.js_files()
+            dyn = m.dynamic_script()
+            if dyn is not None and dyn.strip(" \r\n") != "":
+                dyn += "\n\n"
+                line_cnt = len(dyn.split("\n")) - 1
+                self.script_trace[f"dyn_{m.safe_name()}.js"] = (offset, offset + line_cnt)
+                offset += line_cnt
+                src += dyn
 
         if os.path.isfile(Configuration.frida_scripts):
             files_js += [Configuration.frida_scripts]
@@ -219,9 +212,16 @@ class Fusion(object):
                 sys.exit(1)
 
     def attach(self, pid: int):
-        self.running = True
+        Fusion.running = True
         self.pid = pid
 
+        if self.session is not None:
+            try:
+                self.session.off("detached", self.on_detached)
+            except:
+                pass
+            self.session = None
+
         self.session = self.device.attach(self.pid)
         self.session.on("detached", self.on_detached)
 
@@ -230,7 +230,14 @@ class Fusion(object):
         self.device.resume(self.pid)
 
     def std_spawn(self):
-        self.running = True
+        Fusion.running = True
+
+        if self.session is not None:
+            try:
+                self.session.off("detached", self.on_detached)
+            except:
+                pass
+            self.session = None
 
         self.pid = self.device.spawn([Configuration.package])
         self.session = self.device.attach(self.pid)
@@ -241,7 +248,14 @@ class Fusion(object):
         self.device.resume(self.pid)
 
     def wait_spawn(self):
-        self.running = True
+        Fusion.running = True
+
+        if self.session is not None:
+            try:
+                self.session.off("detached", self.on_detached)
+            except:
+                pass
+            self.session = None
 
         self.pid = self.device.spawn([Configuration.package])
         self.device.resume(self.pid)
@@ -293,15 +307,20 @@ class Fusion(object):
                     mType = jData.get('type', '').lower()
                     mLevel = jData.get('level', None)
                     if mType == "message":
+
+                        if script_location.file_name in self._ignore_messages:
+                            return
+
                         msg = jData.get('message', '')
                         try:
                             msg = base64.b64decode(msg).decode("UTF-8")
                         except:
                             pass
-                        self.print_message(mLevel, msg, script_location=script_location)
+
+                        self.print_message_inst(mLevel, msg, script_location=script_location)
 
                     elif mType == "key_value_data":
-                        self.print_message("V", "RAW JSON:\n    %s" % (
+                        self.print_message_inst("V", "RAW JSON:\n    %s" % (
                             json.dumps(jData, indent=4).replace("\n", "\n    ")
                         ), script_location=script_location)
 
@@ -331,7 +350,7 @@ class Fusion(object):
 
                     # Legacy
                     elif mType == "data":
-                        self.print_message("V", "RAW JSON:\n    %s" % (
+                        self.print_message_inst("V", "RAW JSON:\n    %s" % (
                             json.dumps(jData, indent=4).replace("\n", "\n    ")
                         ), script_location=script_location)
 
@@ -355,15 +374,27 @@ class Fusion(object):
 
                     elif mType == "java-uncaught":
                         self.insert_history('frida', json.dumps(jData))
-                        self.print_message("E", jData.get('stack', ''), script_location=script_location)
+                        self.print_message_inst("E", jData.get('stack', ''), script_location=script_location)
 
                     else:
-                        self.print_message(mLevel, message, script_location=script_location)
+                        self.print_message_inst(mLevel, message, script_location=script_location)
+
+                except SilentKillError as sk:
+                    skm = str(sk)
+
+                    self.print_message_inst("D", "Silent kill requested",
+                                       script_location=Logger.get_caller_info(stack_index=1))
+                    Fusion.running = False
+                    time.sleep(0.2)
+                    if skm != "":
+                        Logger.pl(f'\n{skm}')
+                    Logger.pl('\n{+} {O}Exiting...{O}{W}')
+                    self.done.set()
 
                 except Exception as err:
                     script_location = ScriptLocation(file_name=Fusion._script_name)
-                    self.print_message("E", message, script_location=script_location)
-                    self.print_message("E", payload, script_location=script_location)
+                    self.print_message_inst("E", message, script_location=script_location)
+                    self.print_message_inst("E", payload, script_location=script_location)
                     self.print_exception(err)
 
             else:
@@ -377,7 +408,6 @@ class Fusion(object):
                         stack = "Stack trace:\n"
                         stack += message.get('stack', '')
 
-                        pattern = re.compile(r'(fusion_bundle\.js):(\d+)')
                         matches = [
                             (
                                 m.group(0),
@@ -386,7 +416,7 @@ class Fusion(object):
                                     line=m.group(2),
                                 ))
                             )
-                            for m in pattern.finditer(stack)
+                            for m in Fusion._bundle_pattern.finditer(stack)
                         ]
                         for m in matches:
                             stack = stack.replace(m[0], f"{m[1].file_name}:{m[1].line}")
@@ -405,18 +435,19 @@ class Fusion(object):
                             "stack": stack
                         }))
 
-                        self.print_message("F", description + stack,
+                        self.print_message_inst("F", description + stack,
                                            script_location=script_location)
                         Fusion.running = False
                         time.sleep(0.2)
                         Logger.pl('\n{+} {O}Exiting...{O}{W}')
                         self.done.set()
                     else:
-                        self.print_message("I", message, script_location=script_location)
-                        self.print_message("I", payload, script_location=script_location)
-                except:
-                    self.print_message("I", message, script_location=script_location)
-                    self.print_message("I", payload, script_location=script_location)
+                        self.print_message_inst("I", message, script_location=script_location)
+                        self.print_message_inst("I", payload, script_location=script_location)
+                except Exception as e:
+                    self.print_message_inst("I", message, script_location=script_location)
+                    self.print_message_inst("I", payload, script_location=script_location)
+                    self.print_exception(e)
 
         return handler
 
@@ -431,8 +462,30 @@ class Fusion(object):
         elif crash is not None:
             Logger.pl("[CRASH] details: " + str(crash))
 
+        Logger.pl("")
         self.done.set()
 
+
+    def _replace_location(self, message: str) -> str:
+        try:
+            matches = [
+                (
+                    m.group(0),
+                    self.translate_location(dict(
+                        file_name=m.group(1),
+                        line=m.group(2),
+                    ))
+                )
+                for m in Fusion._bundle_pattern.finditer(message)
+            ]
+            for m in matches:
+                message = message.replace(m[0], f"{m[1].file_name}:{m[1].line}")
+        except Exception as e:
+            print(e)
+            pass
+
+        return message
+
     def _raise_key_value_event(self,
                                script_location: ScriptLocation = None,
                                stack_trace: str = None,
@@ -446,9 +499,11 @@ class Fusion(object):
                     module=module,
                     received_data=received_data
                 )
+            except SilentKillError as ske:
+                raise ske
             except Exception as e:
                 if Configuration.debug_level >= 2:
-                    self.print_message("E", f"Error resizing event to module {m.name}: {str(e)}")
+                    self.print_message_inst("E", f"Error resizing event to module {m.name}: {str(e)}")
                 else:
                     self.print_exception(e)
 
@@ -463,12 +518,51 @@ class Fusion(object):
                     stack_trace=stack_trace,
                     received_data=received_data
                 )
+            except SilentKillError as ske:
+                raise ske
             except Exception as e:
                 if Configuration.debug_level >= 2:
-                    self.print_message("E", f"Error resizing event to module {m.name}: {str(e)}")
+                    self.print_message_inst("E", f"Error resizing event to module {m.name}: {str(e)}")
                 else:
                     self.print_exception(e)
 
+    def print_message_inst(self, level: str = "*", message: str = "",
+                      script_location: ScriptLocation = None):
+
+        return type(self)._print_message(
+            level=level,
+            message=self._replace_location(message),
+            script_location=script_location
+            )
+
+    @classmethod
+    def print_message(cls, level: str = "*", message: str = "",
+                      script_location: ScriptLocation = None):
+        return cls._print_message(
+            level=level,
+            message=message,
+            script_location=script_location
+            )
+
+    @classmethod
+    def _print_message(cls, level: str = "*", message: str = "",
+                      script_location: ScriptLocation = None):
+
+        if Fusion.running is False and Logger.debug_level >= 2:
+            return
+
+        if script_location is None:
+            script_location = ScriptLocation(
+                file_name=Fusion._script_name
+            )
+
+        Logger.print_message(
+            level=level,
+            message=message,
+            script_location=script_location,
+            filename_col_len=Fusion.max_filename
+        )
+
     @classmethod
     def insert_history(cls,  source: str, data: str, stack_trace: str = ''):
         Fusion._db_jobs.put(dict(
@@ -621,7 +715,24 @@ class Fusion(object):
             if len(self._modules) > 0:
                 Logger.pl("{+} Starting selected modules")
                 for m in self._modules:
-                    m.start_module(db_path=Configuration.db_path)
+                    m.load_from_arguments(Configuration.args)
+                    m.start_module(
+                        package=Configuration.package,
+                        db_path=Configuration.db_path
+                    )
+                    if m.safe_name() in Configuration.ignore_messages_modules.keys():
+                        m.suppress_messages()
+
+                self._ignore_messages = [
+                    Path(f).name
+                    for _, md in Configuration.ignore_messages_modules.items()
+                    if (m := next(iter([
+                        mi
+                        for mi in self._modules
+                        if mi.name == md.name
+                    ]), None)) is not None
+                    for f in m.js_files()
+                ]
 
             self.get_device()
             if self.device is not None:
